import { Subject, Observable, Subscription } from 'rxjs';
import { Filter, Event, EventWithAnyFields, EventWithResult, EventMatcher, Listener, ListenerConfig } from './types';
export declare class Channel {
    private eventChannel;
    private resets;
    private filters;
    errors: Subject<string | Error>;
    constructor();
    trigger<T>(eventOrType: string | (EventWithAnyFields & T), payload?: T): EventWithResult;
    filter<T extends Event>(eventMatcher: EventMatcher, f: Filter<T>): Subscription;
    listen<T extends Event, U>(eventMatcher: EventMatcher, listener: Listener<T, U>, config?: ListenerConfig): Subscription;
    on<T extends Event, U>(eventMatcher: EventMatcher, listener: Listener<T, U>, config?: ListenerConfig): Subscription;
    /**
     * Provides an Observable of matching events from the channel.
     */
    query<T extends Event>(eventMatcher: EventMatcher, payloadMatcher?: Object): Observable<T>;
    /** TODO Method doc #spy */
    spy<T extends Event>(spyFn: Filter<T>): Subscription;
    /**
     * TODO Method docs
     */
    reset(): void;
}
export declare const channel: Channel;
export declare const trigger: <T>(eventOrType: string | (EventWithAnyFields & T), payload?: T | undefined) => EventWithResult;
export declare const query: <T extends Event>(eventMatcher: EventMatcher, payloadMatcher?: Object | undefined) => Observable<T>;
export declare const filter: <T extends Event>(eventMatcher: EventMatcher, f: Filter<T>) => Subscription;
export declare const listen: <T extends Event, U>(eventMatcher: EventMatcher, listener: Listener<T, U>, config?: ListenerConfig) => Subscription;
export declare const on: <T extends Event, U>(eventMatcher: EventMatcher, listener: Listener<T, U>, config?: ListenerConfig) => Subscription;
export declare const spy: <T extends Event>(spyFn: Filter<T>) => Subscription;
export declare const reset: () => void;
/** Decorates a function so that its argument is the mutable array
 * of all events seen during its run. Useful for testing:
 *
 * it('does awesome', captureEvents(async seen => {
 *  trigger('foo)
 *  expect(seen).toEqual([{type: 'foo'}])
 * }));
 */
export declare function captureEvents<T>(testFn: (arg: T[]) => void | Promise<any>): () => any;
